def smallr2n(roman_str):
    roman_nums = {'I': 1, 'V': 5, 'X': 10, 'L': 50, 'C': 100, 'D': 500, 'M': 1000, "": 0}
    roman_exps = {'I': 0, 'V': 0, 'X': 1, 'L': 1, 'C': 2, 'D': 2, 'M': 3, "": float('inf')}
    # TODO: Could the roman symbol definitions be made package-wide instead of redefining them per module?
    # Parse roman numerals to 1-4999
    # Assume valid input
    num = 0
    aux = 0
    prev_c = ""

    for c in roman_str:
        # Check changes in exponent to detect decimal placement vs. subtractive notation.
        if roman_exps[c] < roman_exps[prev_c]:
            # Previous block should be handled now
            num += aux
            aux = roman_nums[c]

        elif roman_exps[c] > roman_exps[prev_c]:
            # Encountered subtractive notation.
            aux = roman_nums[c] - aux

        else:  # Same exponent
            if roman_nums[c] <= roman_nums[prev_c]:
                aux += roman_nums[c]  # Add up repeating chars
            else:  # Encountered Subtractive notation
                aux = roman_nums[c] - aux

        prev_c = c

    return num + aux

    # Maybe a state machine?


def r2n(roman_str):
    num = 0
    vin_mul = 1000
    vin_exp = 0
    starti = 0
    sign = 1

    if roman_str == "nulla":
        return 0

    if roman_str[0] == '-':  # Negative number encountered
        sign = -1
        roman_str = roman_str[1:]

    if roman_str[0] == '(':  # Encountered parens/vinculus notation
        for i, c in enumerate(roman_str):
            if c == '(':
                vin_exp += 1
                starti = i + 1
            elif c == ')':
                parse_s = roman_str[starti:i]
                num += vin_mul ** vin_exp * smallr2n(parse_s)
                vin_exp -= 1
                starti = i + 1
                if vin_exp == 0:
                    num += smallr2n(roman_str[i + 1:])  # Handle last block outside parens e.g.. ...)XII
                    break
            else:
                pass

        return sign * num
    else:  # Smaller number.
        return sign * smallr2n(roman_str)



# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open("README.md", "r") as fh:
    long_desc = fh.read()

setup(
    description='Roman numeral to/from converter scripts',
    long_description=long_desc,
    long_description_content_type="text/markdown",
    author='Eetu Lampsijärvi',
    url='https://github.com/Tulitomaatti/romanum',
    download_url='https://github.com/Tulitomaatti/romanum',
    author_email='eetu.lampsijarvi@helsinki.fi',
    version='0.1',
    install_requires=['nose'],
    packages=['romanum'],
    scripts=['bin/n2r', 'bin/r2n'],
    name='romanum',
    classifiers=[
        "Programming Language :: Python :: 2",
    ]
)



# TODO: replace concatenations with string ''.join()s of lists.
# TODO: input validation? Validate input in bin scripts.


def build_roman_digit(digit, exponent=0):
    romanchars = [('I', 'V'), ('X', 'L'), ('C', 'D'), ('M', ''), ('', '')]
    # Using a dict here instead should allow using the same code below even if a
    # more exotic system with more symbols than 1&5 or non-decimal bases is used.
    # This would also make the code neater with rc[exp][digit] style where we
    # don't need to worry about whether the 1 & 5 symbols are the 0 and 1 indices.
    rc = romanchars
    e = exponent

    if digit == 0:
        return ''
    elif digit < 4:
        return digit * rc[e][0]
    elif digit == 4:
        if e == 3:
            return digit * rc[e][0]  # Special case for 4000 MMMM, since no symbol for 5000 exists.
        else:
            return rc[e][0] + rc[e][1]
    elif digit < 9:
        return rc[e][1] + (digit - 5) * rc[e][0]
    else:  # digit == 9 (or more, if input is bad)
        return rc[e][0] + rc[e + 1][0]


def smallnum2r(num):
    # Validate input: Don't; let n2r/scripts handle this.
    # Get thousands/hundreds/tens/one digits: obvious math method
    th = num / 1000
    hu = (num - th * 1000) / 100
    te = (num - th * 1000 - hu * 100) / 10
    on = num - th * 1000 - hu * 100 - te * 10
    # Alt: loop through % 10 and //= 10 on every loop
    # Alt: with string input: loop through string chars after trimming/cleaning.

    retstr = ""

    for digit, exponent in zip([th, hu, te, on], [3, 2, 1, 0]):
        retstr += build_roman_digit(digit, exponent)
    return retstr


def n2r(num):
    sign_prefix = ""
    if num < 0:
        sign_prefix = "-"
        num = abs(num)  # Conversion here will cause the - prefix to appear only at first level.

    if num == 0:
        return "nulla"  # Can't think of a better way here.
    elif num < 5000:
        return sign_prefix + smallnum2r(num)
    else:
        aux = num % 5000
        return sign_prefix + "(" + n2r((num - aux) / 1000) + ")" + smallnum2r(aux)
